﻿local Engine, Api, Locale, Settings = unpack(Immersive)

local Default = Engine:GetModule("Default")
local IsTrackerUpdated = false

-- Registers the element.
function Default:RegisterObjectiveTrackerFrame()
	local settings = Settings:Get(self.Name).Elements["ObjectiveTrackerFrame"]
	local tracker = Api:RegisterElement("ObjectiveTrackerFrame", settings)
	
	-- Add frames
	tracker:AddFrame(ObjectiveTrackerFrame)
	
	-- Add conditions
	tracker:AddConditionRange(settings.Conditions)	
	tracker:AddCondition(function() return IsTrackerUpdated end)

	tracker.OnAfterUpdate = function(self, frame)
		-- Report no tracker update once frame is fully faded in
		if frame == ObjectiveTrackerFrame and frame:GetAlpha() == 1 then
			IsTrackerUpdated = false
		end
	end

	-- Add hotspots
	tracker:AddHotspot(ObjectiveTrackerFrame:GetRect())

	self:RegisterEvent("QUEST_ACCEPT_CONFIRM", "OnQuestUpdate")
	self:RegisterEvent("QUEST_WATCH_UPDATE", "OnQuestUpdate")
	self:RegisterEvent("QUEST_WATCH_LIST_CHANGED", "OnQuestUpdate")
	self:RegisterEvent("QUEST_WATCH_OBJECTIVES_CHANGED", "OnQuestUpdate")
end

-- Unregisters the element.
function Default:UnregisterObjectiveTrackerFrame()
	self:UnregisterEvent("QUEST_WATCH_OBJECTIVES_CHANGED")
	self:UnregisterEvent("QUEST_WATCH_LIST_CHANGED")
	self:UnregisterEvent("QUEST_WATCH_UPDATE")
	self:UnregisterEvent("QUEST_ACCEPT_CONFIRM")

	Api:UnregisterElement("ObjectiveTrackerFrame")
end

-- Occurs when a quest event is fired.
function Default:OnQuestUpdate(event)
	IsTrackerUpdated = true
end